"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var os = require("os");
var Raven = require("raven");
var Promise = require("bluebird");
var common_tags_1 = require("common-tags");
var patterns = require("./utils/patterns");
var captureException = Promise.promisify(Raven.captureException, { context: Raven });
function hasCode(error) {
    return error.code != null;
}
function treatFailedBindingAsMissingModule(error) {
    if (error.message.startsWith('Could not locate the bindings file.')) {
        error.code = 'MODULE_NOT_FOUND';
    }
}
function interpret(error) {
    if (!(error instanceof Error)) {
        return;
    }
    treatFailedBindingAsMissingModule(error);
    if (hasCode(error)) {
        var errorCodeHandler = messages[error.code];
        var message = errorCodeHandler && errorCodeHandler(error);
        if (message) {
            return message;
        }
        if (!_.isEmpty(error.message)) {
            return error.code + ": " + error.message;
        }
    }
    else {
        return error.message;
    }
}
var messages = {
    EISDIR: function (error) { return "File is a directory: " + error.path; },
    ENOENT: function (error) { return "No such file or directory: " + error.path; },
    ENOGIT: function () { return common_tags_1.stripIndent(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n\t\tGit is not installed on this system.\n\t\tHead over to http://git-scm.com to install it and run this command again."], ["\n\t\tGit is not installed on this system.\n\t\tHead over to http://git-scm.com to install it and run this command again."]))); },
    EPERM: function () { return common_tags_1.stripIndent(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n\t\tYou don't have enough privileges to run this operation.\n\t\t", "\n\n\t\tIf this is not the case, and you're trying to burn an SDCard, check that the write lock is not set."], ["\n\t\tYou don't have enough privileges to run this operation.\n\t\t",
        "\n\n\t\tIf this is not the case, and you're trying to burn an SDCard, check that the write lock is not set."])), os.platform() === 'win32'
        ? 'Run a new Command Prompt as administrator and try running this command again.'
        : 'Try running this command again prefixing it with `sudo`.'); },
    EACCES: function (e) { return messages.EPERM(e); },
    ETIMEDOUT: function () {
        return 'Oops something went wrong, please check your connection and try again.';
    },
    MODULE_NOT_FOUND: function () { return common_tags_1.stripIndent(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n\t\tPart of the CLI could not be loaded. This typically means your CLI install is in a broken state.\n\t\t", "\n\t"], ["\n\t\tPart of the CLI could not be loaded. This typically means your CLI install is in a broken state.\n\t\t",
        "\n\t"])), os.arch() === 'x64'
        ? 'You can normally fix this by uninstalling and reinstalling the CLI.'
        : common_tags_1.stripIndent(templateObject_4 || (templateObject_4 = __makeTemplateObject(["\n\t\t\t\tYou're using an unsupported architecture (", "), so this is typically caused by missing native modules.\n\t\t\t\tReinstalling may help, but pay attention to errors in native module build steps en route.\n\t\t\t"], ["\n\t\t\t\tYou're using an unsupported architecture (", "), so this is typically caused by missing native modules.\n\t\t\t\tReinstalling may help, but pay attention to errors in native module build steps en route.\n\t\t\t"])), os.arch())); },
    ResinExpiredToken: function () { return common_tags_1.stripIndent(templateObject_5 || (templateObject_5 = __makeTemplateObject(["\n\t\tLooks like your session token is expired.\n\t\tPlease try logging in again with:\n\t\t\t$ resin login"], ["\n\t\tLooks like your session token is expired.\n\t\tPlease try logging in again with:\n\t\t\t$ resin login"]))); },
};
exports.handle = function (error) {
    var message = interpret(error);
    if (message == null) {
        return;
    }
    if (process.env.DEBUG) {
        message = error.stack;
    }
    patterns.printErrorMessage(message);
    return captureException(error)
        .timeout(1000)
        .catch(function () {
    })
        .finally(function () { return process.exit(error.exitCode || 1); });
};
var templateObject_1, templateObject_2, templateObject_4, templateObject_3, templateObject_5;
//# sourceMappingURL=errors.js.map