"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var Promise = require("bluebird");
var ResinSdk = require("resin-sdk");
var deviceConfig = require("resin-device-config");
var semver = require("resin-semver");
var resin = ResinSdk.fromSharedOptions();
function readRootCa() {
    var caFile = process.env.NODE_EXTRA_CA_CERTS;
    if (!caFile) {
        return Promise.resolve();
    }
    return Promise.fromCallback(function (cb) {
        return fs.readFile(caFile, { encoding: 'utf8' }, cb);
    })
        .then(function (pem) { return Buffer.from(pem).toString('base64'); })
        .catch({ code: 'ENOENT' }, function () { });
}
function generateBaseConfig(application, options) {
    if (options.appUpdatePollInterval) {
        options = __assign({}, options, { appUpdatePollInterval: options.appUpdatePollInterval * 60 * 1000 });
    }
    return Promise.props({
        userId: resin.auth.getUserId(),
        username: resin.auth.whoami(),
        apiUrl: resin.settings.get('apiUrl'),
        vpnUrl: resin.settings.get('vpnUrl'),
        registryUrl: resin.settings.get('registryUrl'),
        deltaUrl: resin.settings.get('deltaUrl'),
        apiConfig: resin.models.config.getAll(),
        rootCA: readRootCa().catch(function () {
            console.warn('Could not read root CA');
        }),
    }).then(function (results) {
        return deviceConfig.generate({
            application: application,
            user: {
                id: results.userId,
                username: results.username,
            },
            endpoints: {
                api: results.apiUrl,
                vpn: results.vpnUrl,
                registry: results.registryUrl,
                delta: results.deltaUrl,
            },
            pubnub: results.apiConfig.pubnub,
            mixpanel: {
                token: results.apiConfig.mixpanelToken,
            },
            balenaRootCA: results.rootCA,
        }, options);
    });
}
exports.generateBaseConfig = generateBaseConfig;
function generateApplicationConfig(application, options) {
    return generateBaseConfig(application, options).tap(function (config) {
        if (semver.satisfies(options.version, '>=2.7.8')) {
            return addProvisioningKey(config, application.id);
        }
        else {
            return addApplicationKey(config, application.id);
        }
    });
}
exports.generateApplicationConfig = generateApplicationConfig;
function generateDeviceConfig(device, deviceApiKey, options) {
    return resin.models.application
        .get(device.belongs_to__application.__id)
        .then(function (application) {
        return generateBaseConfig(application, options).tap(function (config) {
            if (deviceApiKey) {
                return addDeviceKey(config, device.uuid, deviceApiKey);
            }
            else if (semver.satisfies(options.version, '>=2.0.3')) {
                return addDeviceKey(config, device.uuid, true);
            }
            else {
                return addApplicationKey(config, application.id);
            }
        });
    })
        .then(function (config) {
        config.registered_at = Math.floor(Date.now() / 1000);
        config.deviceId = device.id;
        config.uuid = device.uuid;
        return config;
    });
}
exports.generateDeviceConfig = generateDeviceConfig;
function addApplicationKey(config, applicationNameOrId) {
    return resin.models.application
        .generateApiKey(applicationNameOrId)
        .tap(function (apiKey) {
        config.apiKey = apiKey;
    });
}
function addProvisioningKey(config, applicationNameOrId) {
    return resin.models.application
        .generateProvisioningKey(applicationNameOrId)
        .tap(function (apiKey) {
        config.apiKey = apiKey;
    });
}
function addDeviceKey(config, uuid, customDeviceApiKey) {
    return Promise.try(function () {
        if (customDeviceApiKey === true) {
            return resin.models.device.generateDeviceKey(uuid);
        }
        else {
            return customDeviceApiKey;
        }
    }).tap(function (deviceApiKey) {
        config.deviceApiKey = deviceApiKey;
    });
}
//# sourceMappingURL=config.js.map