"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var os = require("os");
var Promise = require("bluebird");
var _ = require("lodash");
var chalk_1 = require("chalk");
var rindle = require("rindle");
var visuals = require("resin-cli-visuals");
var ResinSdk = require("resin-sdk");
var waitStreamAsync = Promise.promisify(rindle.wait);
var resin = ResinSdk.fromSharedOptions();
function getGroupDefaults(group) {
    return _.chain(group)
        .get('options')
        .map(function (question) { return [question.name, question.default]; })
        .fromPairs()
        .value();
}
exports.getGroupDefaults = getGroupDefaults;
function stateToString(state) {
    var percentage = _.padStart("" + state.percentage, 3, '0');
    var result = chalk_1.default.blue(percentage + '%') + " " + chalk_1.default.cyan(state.operation.command);
    switch (state.operation.command) {
        case 'copy':
            return result + " " + state.operation.from.path + " -> " + state.operation.to.path;
        case 'replace':
            return result + " " + state.operation.file.path + ", " + state.operation.copy + " -> " + state.operation.replace;
        case 'run-script':
            return result + " " + state.operation.script;
        default:
            throw new Error("Unsupported operation: " + state.operation.command);
    }
}
exports.stateToString = stateToString;
function sudo(command, _a) {
    var _b = _a === void 0 ? {} : _a, stderr = _b.stderr, msg = _b.msg;
    var executeWithPrivileges = require('./sudo').executeWithPrivileges;
    if (os.platform() !== 'win32') {
        console.log(msg || 'If asked please type your computer password to continue');
    }
    return executeWithPrivileges(command, stderr);
}
exports.sudo = sudo;
function runCommand(command) {
    var capitano = require('capitano');
    return Promise.fromCallback(function (resolver) { return capitano.run(command, resolver); });
}
exports.runCommand = runCommand;
function getManifest(image, deviceType) {
    var imagefs = require('resin-image-fs');
    return imagefs
        .readFile({
        image: image,
        partition: 1,
        path: '/device-type.json',
    })
        .then(JSON.parse)
        .catch(function () { return resin.models.device.getManifestBySlug(deviceType); });
}
exports.getManifest = getManifest;
function osProgressHandler(step) {
    step.on('stdout', process.stdout.write.bind(process.stdout));
    step.on('stderr', process.stderr.write.bind(process.stderr));
    step.on('state', function (state) {
        if (state.operation.command === 'burn') {
            return;
        }
        console.log(exports.stateToString(state));
    });
    var progressBars = {
        write: new visuals.Progress('Writing Device OS'),
        check: new visuals.Progress('Validating Device OS'),
    };
    step.on('burn', function (state) { return progressBars[state.type].update(state); });
    return waitStreamAsync(step);
}
exports.osProgressHandler = osProgressHandler;
function getArchAndDeviceType(applicationName) {
    return Promise.join(getApplication(applicationName), resin.models.config.getDeviceTypes(), function (app, deviceTypes) {
        var config = _.find(deviceTypes, { slug: app.device_type });
        if (!config) {
            throw new Error('Could not read application information!');
        }
        return { device_type: app.device_type, arch: config.arch };
    });
}
exports.getArchAndDeviceType = getArchAndDeviceType;
function getApplication(applicationName) {
    var match = applicationName.split('/');
    var extraOptions = {
        $expand: {
            application_type: {
                $select: ['name', 'slug', 'supports_multicontainer', 'is_legacy'],
            },
        },
    };
    if (match.length > 1) {
        return resin.models.application.getAppByOwner(match[1], match[0], extraOptions);
    }
    return resin.models.application.get(applicationName, extraOptions);
}
exports.getApplication = getApplication;
function getSubShellCommand(command) {
    if (os.platform() === 'win32') {
        return {
            program: 'cmd.exe',
            args: ['/s', '/c', command],
        };
    }
    else {
        return {
            program: '/bin/sh',
            args: ['-c', command],
        };
    }
}
exports.getSubShellCommand = getSubShellCommand;
//# sourceMappingURL=helpers.js.map