"use strict";
var os_1 = require("os");
var _ = require("lodash");
var chalk_1 = require("chalk");
var resin_stream_logger_1 = require("resin-stream-logger");
var Logger = (function () {
    function Logger() {
        var logger = new resin_stream_logger_1.StreamLogger();
        logger.addPrefix('build', chalk_1.default.blue('[Build]'));
        logger.addPrefix('info', chalk_1.default.cyan('[Info]'));
        logger.addPrefix('debug', chalk_1.default.magenta('[Debug]'));
        logger.addPrefix('success', chalk_1.default.green('[Success]'));
        logger.addPrefix('warn', chalk_1.default.yellow('[Warn]'));
        logger.addPrefix('error', chalk_1.default.red('[Error]'));
        this.streams = {
            build: logger.createLogStream('build'),
            info: logger.createLogStream('info'),
            debug: logger.createLogStream('debug'),
            success: logger.createLogStream('success'),
            warn: logger.createLogStream('warn'),
            error: logger.createLogStream('error'),
        };
        _.forEach(this.streams, function (stream, key) {
            if (key !== 'debug' || process.env.DEBUG) {
                stream.pipe(process.stdout);
            }
        });
        this.formatMessage = logger.formatWithPrefix.bind(logger);
    }
    Logger.prototype.logInfo = function (msg) {
        return this.streams.info.write(msg + os_1.EOL);
    };
    Logger.prototype.logDebug = function (msg) {
        return this.streams.debug.write(msg + os_1.EOL);
    };
    Logger.prototype.logSuccess = function (msg) {
        return this.streams.success.write(msg + os_1.EOL);
    };
    Logger.prototype.logWarn = function (msg) {
        return this.streams.warn.write(msg + os_1.EOL);
    };
    Logger.prototype.logError = function (msg) {
        return this.streams.error.write(msg + os_1.EOL);
    };
    return Logger;
}());
module.exports = Logger;
//# sourceMappingURL=logger.js.map