"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var JSONStream = require("JSONStream");
var request = require("request");
var compose_1 = require("./compose");
var DEBUG_MODE = !!process.env.DEBUG;
var CURSOR_METADATA_REGEX = /([a-z]+)([0-9]+)?/;
var TRIM_REGEX = /\n+$/;
function getBuilderEndpoint(baseUrl, owner, app, opts) {
    return __awaiter(this, void 0, void 0, function () {
        var querystring, args;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Promise.resolve().then(function () { return require('querystring'); })];
                case 1:
                    querystring = _a.sent();
                    args = querystring.stringify({
                        owner: owner,
                        app: app,
                        emulated: opts.emulated,
                        nocache: opts.nocache,
                    });
                    return [2, "https://builder." + baseUrl + "/v3/build?" + args];
            }
        });
    });
}
function startRemoteBuild(build) {
    return __awaiter(this, void 0, void 0, function () {
        var Bluebird, stream, readline;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Promise.resolve().then(function () { return require('bluebird'); })];
                case 1:
                    Bluebird = _a.sent();
                    return [4, getRequestStream(build)];
                case 2:
                    stream = _a.sent();
                    if (!(process.platform === 'win32')) return [3, 4];
                    return [4, Promise.resolve().then(function () { return require('readline'); })];
                case 3:
                    readline = (_a.sent()).createInterface({
                        input: process.stdin,
                        output: process.stdout,
                    });
                    readline.on('SIGINT', function () { return process.emit('SIGINT'); });
                    _a.label = 4;
                case 4: return [2, new Bluebird(function (resolve, reject) {
                        process.removeAllListeners('SIGINT');
                        process.on('SIGINT', function () {
                            process.stderr.write('Received SIGINT, cleaning up. Please wait.\n');
                            cancelBuildIfNecessary(build).then(function () {
                                stream.end();
                                process.exit(130);
                            });
                        });
                        stream.on('data', getBuilderMessageHandler(build));
                        stream.on('end', resolve);
                        stream.on('error', reject);
                    }).return()];
            }
        });
    });
}
exports.startRemoteBuild = startRemoteBuild;
function handleBuilderMetadata(obj, build) {
    return __awaiter(this, void 0, void 0, function () {
        var stripIndent, _a, readline, match, value, amount;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4, Promise.resolve().then(function () { return require('common-tags'); })];
                case 1:
                    stripIndent = (_b.sent()).stripIndent;
                    _a = obj.resource;
                    switch (_a) {
                        case 'cursor': return [3, 2];
                        case 'buildLogId': return [3, 4];
                    }
                    return [3, 5];
                case 2: return [4, Promise.resolve().then(function () { return require('readline'); })];
                case 3:
                    readline = _b.sent();
                    if (obj.value == null) {
                        return [2];
                    }
                    match = obj.value.match(CURSOR_METADATA_REGEX);
                    if (!match) {
                        console.log(stripIndent(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n\t\t\t\t\tWarning: ignoring unknown builder command. You may experience\n\t\t\t\t\todd build output. Maybe you need to update resin-cli?"], ["\n\t\t\t\t\tWarning: ignoring unknown builder command. You may experience\n\t\t\t\t\todd build output. Maybe you need to update resin-cli?"]))));
                        return [2];
                    }
                    value = match[1];
                    amount = match[2] || 1;
                    switch (value) {
                        case 'erase':
                            readline.clearLine(process.stdout, 0);
                            process.stdout.write('\r');
                            break;
                        case 'up':
                            readline.moveCursor(process.stdout, 0, -amount);
                            break;
                        case 'down':
                            readline.moveCursor(process.stdout, 0, amount);
                            break;
                    }
                    return [3, 5];
                case 4:
                    build.releaseId = parseInt(obj.value, 10);
                    return [3, 5];
                case 5: return [2];
            }
        });
    });
}
function getBuilderMessageHandler(build) {
    var _this = this;
    return function (obj) { return __awaiter(_this, void 0, void 0, function () {
        var readline, message;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (DEBUG_MODE) {
                        console.log("[debug] handling message: " + JSON.stringify(obj));
                    }
                    if (obj.type != null && obj.type === 'metadata') {
                        return [2, handleBuilderMetadata(obj, build)];
                    }
                    if (!obj.message) return [3, 2];
                    return [4, Promise.resolve().then(function () { return require('readline'); })];
                case 1:
                    readline = _a.sent();
                    readline.clearLine(process.stdout, 0);
                    message = obj.message.replace(TRIM_REGEX, '');
                    if (obj.replace) {
                        process.stdout.write("\r" + message);
                    }
                    else {
                        process.stdout.write("\r" + message + "\n");
                    }
                    _a.label = 2;
                case 2: return [2];
            }
        });
    }); };
}
function cancelBuildIfNecessary(build) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(build.releaseId != null)) return [3, 2];
                    return [4, build.sdk.pine.patch({
                            resource: 'release',
                            id: build.releaseId,
                            body: {
                                status: 'cancelled',
                                end_timestamp: Date.now(),
                            },
                        })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2: return [2];
            }
        });
    });
}
function getRequestStream(build) {
    return __awaiter(this, void 0, void 0, function () {
        var path, visuals, tarSpinner, tarStream, post, _a, _b, _c, uploadSpinner, parseStream;
        return __generator(this, function (_d) {
            switch (_d.label) {
                case 0: return [4, Promise.resolve().then(function () { return require('path'); })];
                case 1:
                    path = _d.sent();
                    return [4, Promise.resolve().then(function () { return require('resin-cli-visuals'); })];
                case 2:
                    visuals = _d.sent();
                    tarSpinner = new visuals.Spinner('Packaging the project source...');
                    tarSpinner.start();
                    return [4, compose_1.tarDirectory(path.resolve(build.source))];
                case 3:
                    tarStream = _d.sent();
                    tarSpinner.stop();
                    if (DEBUG_MODE) {
                        console.log('[debug] Opening builder connection');
                    }
                    _b = (_a = request).post;
                    _c = {};
                    return [4, getBuilderEndpoint(build.baseUrl, build.owner, build.app, build.opts)];
                case 4:
                    post = _b.apply(_a, [(_c.url = _d.sent(),
                            _c.auth = {
                                bearer: build.auth,
                            },
                            _c)]);
                    uploadSpinner = new visuals.Spinner('Uploading source package to resin cloud');
                    uploadSpinner.start();
                    tarStream.pipe(post);
                    parseStream = post.pipe(JSONStream.parse('*'));
                    parseStream.on('data', function () { return uploadSpinner.stop(); });
                    return [2, parseStream];
            }
        });
    });
}
var templateObject_1;
//# sourceMappingURL=remote-build.js.map